/*********************************************************************
 * Copyright (c) 2012-2012 Integrated Device Technology, Inc.
 * All Rights Reserved
 *
 * This file is provided for use with IDT products only, and does not
 * convey any license, whether express or implied, to any intellectual
 * property rights of Integrated Device Technology, Inc.
 * Distribution, copying or modification of the contents of this file
 * and/or any derivatives thereto is not permitted, except as
 * specifically authorized in writing. This notice shall be included
 * in its entirety on all copies made from this file.
 *
 * The contents herein are the proprietary and confidential
 * information of Integrated Device Technology Inc. and shall not be
 * distributed or furnished to third parties or made public without
 * prior written permission by Integrated Device Technology Inc.
 * Disclosures of this file in any form to the United States
 * Government may not be made without a written predetermination that
 * it will be protected by limited or restricted rights as set forth
 * in the applicable Federal Acquisition Regulations or other
 * equivalent governmental rule or regulation.
 *
 * The contents herein are provided "as is" and without any express or
 * implied warranties, including, without limitation, the implied
 * warranties of merchantability and fitness for a particular purpose.
 *
 * Last modified: <lastCommitDate>
 * Release: <releaseName>
 * Id: <gitCommitHash>
 *
 *********************************************************************/

#include <stdlib.h>

#include "idt82V3910.h"
#include "profiles.h"
#include "inputFreq.h"
#include "outputFreq.h"

/**
 * @brief Sample configuration for 156.25MHz output at port 6
 *
 *      IN3 (25 MHz) -> T0 +-> OUT6 (Ethernet, 156.25 MHz)
 *
 * @param hdlr Device driver handler
 */
void IDTSample_ConfigureSampleConfigOutput6_156_dot_25MHz(T_idtDrvHdlr hdlr)
{
	/* Disable all outputs */
	IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure input 3, 25MHz */
	IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

	/* Configure DPLL for G8262 Option 1 */
	IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

	/* Configure output output 6 (156.25 MHz) using T0 */
	IDTHlApi_ConfigureOutput(hdlr,
							6,
							OutFreq_156250k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL); 

    /* Switch to backup APLL configuration now it's configured 
     * Note: this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_1);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
}

/**
 * @brief Sample configuration for 77.76 MHz output at port 7
 *
 *      IN3 (25 MHz) -> T0 +-> OUT7 (SONET, 77.76 MHz)
 *
 * @param hdlr Device driver handler
 */
 void IDTSample_ConfigureSampleConfigureOutput7_77_dot_76MHz(T_idtDrvHdlr hdlr)
{
	/* Disable all outputs */
	IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure input 3, 25MHz */
	IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

	/* Configure DPLL for GR-253 Stratum 3 */
	IDTHlApi_gr253SonetStratum3(hdlr, DPLL_INSTANCE_1);

	/* Configure output output 7 (77.76 MHz) using T0 */
	IDTHlApi_ConfigureOutput(hdlr,
							7,
							OutFreq_77760k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

    /* Switch to backup APLL configuration now it's configured 
     * Note: this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_1);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
}

/**
 * @brief Sample configuration for 25MHz output at port 10
 *
 *      IN3 (25 MHz) -> T0 +-> OUT10 (Ethernet, 25 MHz)
 *
 * @param hdlr Device driver handler
 */
void IDTSample_ConfigureSampleConfigureOutput10_25_MHz(T_idtDrvHdlr hdlr)
{
	/* Disable all outputs */
	IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure input 3, 25MHz */
	IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

	/* Configure DPLL for G8262 Option 1 */
	IDTHlApi_g8262EecOption1(hdlr, DPLL_INSTANCE_1);

	/* Configure output 10 (25 MHz) using T0 */
	IDTHlApi_ConfigureOutput(hdlr,
							10,
							OutFreq_25000k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

    /* Switch to backup APLL configuration now it's configured 
     * Note: this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_2);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 10, 1);
}

/**
 * @brief Sample configuration for SyncE WAN application
 *
 *      Inputs: IN3 (25MHz), IN5 (156.25MHz), IN14 (1.544MHz)
 *      T0 Profile: G.8262 Option 2
 *      T4 Profile: Default
 *      Outputs: OUT1 (25MHz), OUT6 (25MHz), OUT7 (156.25MHz), OUT9 (1.544MHz)
 *      T0: IN3, IN5, IN14 -> OUT1, OUT10, OUT11 (IN14 has highest priority)
 *      T4: IN3, IN5 -> OUT9
 *      APLL1 XTAL1 at 25MHz
 *
 * @param hdlr Device driver handler
 */
void IDTSample_SyncE_WAN(T_idtDrvHdlr hdlr)
{
	/* Enable automatic selection on both T0 and T4 DPLL */
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_1, 1);
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_2, 1);

	/* Disable all inputs */
	IDTInput_DisableAllInputs(hdlr);

	/* Configure inputs */
	{
		/* Configure input 3 at 25MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

		/* Configure input 5 at 156.25MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 5, 156250);

		/* Configure input 14 at 1.544MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 14, 1544);
	}

	/* Select inputs for T0 DPLL  */
	{
		/* Configure input 3 with default priority */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 4);

		/* Configure input 5 with default priority */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_1, 6);

		/* Configure input 14 with highest priority */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_1, 1);
	}

	/* Select inputs for T4 DPLL */
	{
		/* Configure input 3 with priority 4 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_2, 4);

		/* Configure input 5 with priority 6 */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_2, 6);

		/* Disable input 14 - priority 0 means never lock to this input */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_2, 0);
	}
	
	/* Configure T0 for G8262 Option 2 */
	IDTHlApi_g8262EecOption2(hdlr, DPLL_INSTANCE_1);

    /* Disable all outputs */
	IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure outputs */
	{
		/* Configure output 1 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							1,
							OutFreq_25000k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 6 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							6,
							OutFreq_25000k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 7 (156.25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							7,
							OutFreq_156250k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 9 (1.544MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							9,
							OutFreq_1544k,
							DPLL_INSTANCE_2,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);
	}

    /* Switch to backup APLL configuration now it's configured 
     * Note: this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_1);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 1, 1);
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
    IDTOutput_SetOutputEnable(hdlr, 9, 1);
}

/**
 * @brief Sample configuration for SONET application
 *
 *	Inputs: IN3 (19.44MHz), IN5 (155.52MHz), IN14 (1.544MHz)
 *	T0 Profile: ST3 SONET
 *	T4 Profile: default
 *	Outputs: OUT1 (19.44MHz), OUT9 (1.544MHz), OUT10 (155.52MHz)
 *	T0: IN3, IN5, IN14 -> OUT1, OUT10 (IN14 is highest priority)
 *	T4: IN3, IN5 -> OUT9
 *	APLL2 XTAL2 at 24.8832MHz
 *
 * @param hdlr Device driver handler
 */
void IDTSample_Sonet(T_idtDrvHdlr hdlr)
{
	/* Enable automatic selection on both T0 and T4 DPLL */
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_1, 1);
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_2, 1);

	/* Disable all inputs */
	IDTInput_DisableAllInputs(hdlr);

	/* Configure inputs */
	{
		/* Configure input 3 at 19.44MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 3, 19440);

		/* Configure input 5 at 155.52MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 5, 155520);

		/* Configure input 14 at 1.544MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 14, 1544);
	}

	/* Select inputs for T0 DPLL  */
	{
		/* Configure input 3 with default priority */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 4);

		/* Configure input 5 with default priority */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_1, 6);

		/* Configure input 14 with highest priority */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_1, 1);
	}

	/* Select inputs for T4 DPLL */
	{
		/* Configure input 3 with priority 4 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_2, 4);

		/* Configure input 5 with priority 6 */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_2, 6);

		/* Disable input 14  - priority 0 means never lock to this input */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_2, 0);
	}
	
	/* Configure T0 for ST3 SONET */
	IDTHlApi_gr253SonetStratum3(hdlr, DPLL_INSTANCE_1);

    /* Disable all outputs */
	IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure outputs */
	{
		/* Configure output 1 (19.44MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							1,
							OutFreq_19440k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 10 (155.52MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							10,
							OutFreq_155520k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 9 (1.544MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							9,
							OutFreq_1544k,
							DPLL_INSTANCE_2,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);
	}

    /* Switch to backup APLL configuration now it's configured Note:
     * this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or
     * APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_2);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 1, 1);
    IDTOutput_SetOutputEnable(hdlr, 10, 1);
    IDTOutput_SetOutputEnable(hdlr, 9, 1);
}

/**
 * @brief Sample configuration for dual mode application
 *
 *      Inputs: IN3 (19.44MHz), IN5 (155.52MHz), IN14 (1.544MHz)
 *      T0 Profile: ST3 SONET
 *      T4 Profile: default
 *      Select XTAL3: done automatically by IDTHlApi_ConfigureOutput function
 *                    based on the output frequency. To manually select XTAL,
 *                    call SetJaXtalId function.
 *      Outputs: OUT1 (19.44MHz), OUT6 (155.52MHz), OUT7 (77.76MHz), OUT9 (1.544MHz)
 *      T0: IN3, IN5, IN14 -> OUT1, OUT6, OUT7 (IN14 is highest priority)
 *      T4: IN3, IN5 -> OUT9
 *      APLL1 XTAL1 at 25MHz & XTAL3 at 24.8832MHz
 *
 * @param hdlr Device driver handler
 */
void IDTSample_DualMode(T_idtDrvHdlr hdlr)
{
	/* Enable automatic selection on both T0 and T4 DPLL */
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_1, 1);
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_2, 1);

	/* Disable all inputs */
	IDTInput_DisableAllInputs(hdlr);

	/* Configure inputs */
	{
		/* Configure input 3 at 19.44MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 3, 19440);

		/* Configure input 5 at 155.52MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 5, 155520);

		/* Configure input 14 at 1.544MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 14, 1544);
	}

	/* Select inputs for T0 DPLL  */
	{
		/* Configure input 3 with default priority */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 4);

		/* Configure input 5 with default priority */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_1, 6);

		/* Configure input 14 with highest priority */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_1, 1);
	}

	/* Select inputs for T4 DPLL */
	{
		/* Configure input 3 with priority 4 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_2, 4);

		/* Configure input 5 with priority 6 */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_2, 6);

		/* Disable input 14 - priority 0 means never lock to this input */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_2, 0);
	}
	
	/* Configure T0 for ST3 SONET */
	IDTHlApi_gr253SonetStratum3(hdlr, DPLL_INSTANCE_1);

    /* Disable all outputs */
	IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure outputs */
	{
		/* Configure output 1 (19.44MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							1,
							OutFreq_19440k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 6 (155.52MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							6,
							OutFreq_155520k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 7 (77.76MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							7,
							OutFreq_77760k,
							DPLL_INSTANCE_1,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 9 (1.544MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							9,
							OutFreq_1544k,
							DPLL_INSTANCE_2,
							sonetGigeMode_Matching,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);
	}

    /* Switch to backup APLL configuration now it's configured 
     * Note: this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_1);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 1, 1);
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
    IDTOutput_SetOutputEnable(hdlr, 9, 1);
}

/**
 * @brief Sample configuration for SyncE SONET application
 *
 *	Inputs: IN3 (25MHz) , IN5 (156.25MHz), IN6 (155.52MHz), IN7 (19.44MHz), IN14 (1.544MHz)
 *	T0 Profile: G.8262 Option 2
 *	T4 Profile: BITS / SSU (Wideband) - default
 *	Outputs: OUT1 (25MHz), OUT2 (19.44MHz), OUT6 (25MHz), OUT7 (156.25MHz),
 *	         OUT9 (1.544MHz), OUT10 (155.52MHz), OUT11 (77.76MHz)
 *	T0: IN3, IN5, IN6, IN7, IN14 -> OUT1, OUT2, OUT6, OUT7, OUT10, OUT11 (IN14 is highest priority)
 *	T4: IN3, IN5, IN6, IN7 -> OUT9
 *	APLL1 XTAL1 at 25MHz & APLL2 XTAL2 at 24.8832MHz
 *
 * @param hdlr Device driver handler
 */
void IDTSample_SyncE_Sonet(T_idtDrvHdlr hdlr)
{
	/* Enable automatic selection on both T0 and T4 DPLL */
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_1, 1);
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_2, 1);

	/* Disable all inputs */
	IDTInput_DisableAllInputs(hdlr);

	/* Configure inputs */
	{
		/* Configure input 3 at 25MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

		/* Configure input 5 at 156.25MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 5, 156250);

		/* Configure input 6 at 155.52MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 6, 155520);

		/* Configure input 7 at 19.44MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 7, 19440);

		/* Configure input 14 at 1.544MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 14, 1544);
	}

	/* Configure T0 for ST3 SONET */
	IDTHlApi_gr253SonetStratum3(hdlr, DPLL_INSTANCE_1);

	/* Select inputs for T0 DPLL  */
	{
		/* Configure input 3 with default priority */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 4);

		/* Configure input 5 with default priority */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_1, 6);

		/* Configure input 6 with default priority */
		IDTInput_SetPriority(hdlr, 6, DPLL_INSTANCE_1, 7);

		/* Configure input 7 with default priority */
		IDTInput_SetPriority(hdlr, 7, DPLL_INSTANCE_1, 8);

		/* Configure input 14 with highest priority */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_1, 1);
	}

	/* Select inputs for T4 DPLL */
	{
		/* Configure input 3 with priority 4 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_2, 4);

		/* Configure input 5 with priority 6 */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_2, 6);

		/* Configure input 6 with priority 7 */
		IDTInput_SetPriority(hdlr, 6, DPLL_INSTANCE_2, 7);

		/* Configure input 7 with priority 8 */
		IDTInput_SetPriority(hdlr, 7, DPLL_INSTANCE_2, 8);

		/* Disable input 14 - priority 0 means never lock to this input */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_2, 0);
	}

    /* Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure outputs */
	{
        /* We need both instances of sonet/gigEth mux to be derived
           from DPLL #1 (T0) */

		/* Configure output 1 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							1,
							OutFreq_25000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 2 (19.44MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							2,
							OutFreq_19440k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 6 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							6,
							OutFreq_25000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 7 (156.25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							7,
							OutFreq_156250k,
							DPLL_INSTANCE_MAX,
						    sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 10 (155.52MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							10,
							OutFreq_155520k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 11 (77.76MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							11,
							OutFreq_77760k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 9 (1.544MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							9,
							OutFreq_1544k,
							DPLL_INSTANCE_2,
                            sonetGigeMode_AllFromDpll1, /* Doesn't
                                                           matter
                                                           since
                                                           signal is
                                                           from DPLL
                                                           directly */
							APLL_CONFIG_MAX,
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);
	}

    /* Switch to backup APLL configuration now it's configured 
     * Note: this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_1);
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_2);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 1, 1);
    IDTOutput_SetOutputEnable(hdlr, 2, 1);
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
    IDTOutput_SetOutputEnable(hdlr, 10, 1);
    IDTOutput_SetOutputEnable(hdlr, 11, 1);
    IDTOutput_SetOutputEnable(hdlr, 9, 1);
}

/**
 * @brief Sample configuration for SyncE LAN application
 *
 *	Inputs: IN3 (25MHz), IN14 (2.048MHz)
 *	T0 Profile: G.8262 Option 2
 *	T4 Profile: BITS / SSU (Wideband)
 *	Outputs: OUT1 (25MHz), OUT10 (25.78125), OUT11 (161.1328125MHz), OUT9 (2.048MHz)
 *	T0: IN3, IN14 -> OUT1, OUT10, OUT11 (IN14 is highest priority)
 *	T4: IN3 -> OUT9
 *	APLL2 XTAL4 at 25.78125MHz
 *
 * @param hdlr Device driver handler
 */
void IDTSample_SyncE_LAN(T_idtDrvHdlr hdlr)
{
	/* Enable automatic selection on both T0 and T4 DPLL */
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_1, 1);
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_2, 1);

	/* Disable all inputs */
	IDTInput_DisableAllInputs(hdlr);

	/* Configure inputs */
	{
		/* Configure input 3 at 25MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 3, 25000);

		/* Configure input 14 at 2.048MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 14, 2048);
	}

	/* Select inputs for T0 DPLL  */
	{
		/* Configure input 3 with default priority */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 4);

		/* Configure input 14 with highest priority */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_1, 1);
	}

	/* Select inputs for T4 DPLL */
	{
		/* Configure input 3 with priority 4 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_2, 4);

		/* Disable input 14 - priority 0 means never lock to this input */
		IDTInput_SetPriority(hdlr, 14, DPLL_INSTANCE_2, 0);
	}
	
	/* Configure T0 for G8262 Option 2 */
	IDTHlApi_g8262EecOption2(hdlr, DPLL_INSTANCE_1);

    /* Disable all outputs */
	IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure outputs */
	{
		/* Configure output 1 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							1,
							OutFreq_25000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 10 (25.78125MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							10,
							OutFreq_25000k_66_64,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 11 (161.1328125MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							11,
							OutFreq_156250k_66_64,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 9 (2.048MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							9,
							OutFreq_2048k,
							DPLL_INSTANCE_2,
                            sonetGigeMode_Matching, /* Doesn't matter
                                                       since derived
                                                       from DPLL */
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);
	}

    /* Switch to backup APLL configuration now it's configured 
     * Note: this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_2);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 1, 1);
    IDTOutput_SetOutputEnable(hdlr, 10, 1);
    IDTOutput_SetOutputEnable(hdlr, 11, 1);
    IDTOutput_SetOutputEnable(hdlr, 9, 1);
}

/**
 * @brief Sample configuration for GPS 1PPS application
 *
 *	Inputs: IN3 (1PPS), IN5 (1PPS)
 *	T0 Profile: GPS (15mHz)
 *	Outputs: OUT1 (25MHz), OUT2 (19.44MHz), OUT3 (10MHz), OUT4 (1PPS),
 *	         OUT6 (25MHz), OUT7 (156.25MHz), OUT9 (2.048MHz),
 *	         OUT10 (155.52MHz), OUT11 (77.76MHz)
 *	APLL1 XTAL1 at 25MHz & APLL2 XTAL2 at 24.8832MHz
 *
 * @param hdlr Device driver handler
 */
void IDTSample_GPS_1PPS(T_idtDrvHdlr hdlr)
{
	/* Enable automatic selection on both T0 and T4 DPLL */
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_1, 1);
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_2, 1);

	/* Disable all inputs */
	IDTInput_DisableAllInputs(hdlr);
	
	/* Configure inputs */
	{
		/* Configure input 3, 1 PPS (1 Hz) */
		IDTHlApi_ConfigureInput1PPS(hdlr, 3);

	
		/* Configure input 5, 1 PSS (1 Hz) */
		IDTHlApi_ConfigureInput1PPS(hdlr, 5);
	}

	/* Configure DPLL for 1 PPS */
	IDTHlApi_pps(hdlr, DPLL_INSTANCE_1);

	/* Select inputs for T0 DPLL */
	{
		/* Configure input 3 with priority 4 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 4);

		/* Configure input 5 with priority 6 */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_1, 6);
	}

	/* Select inputs for T4 DPLL */
	{
		/* Disable input 3 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_2, 0);
	
		/* Disable input 5 */
		IDTInput_SetPriority(hdlr, 5, DPLL_INSTANCE_2, 0);
	}

    /* Disable all outputs */
	IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure outputs */
	{
	
		/* Configure output 1 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							1,
							OutFreq_25000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 2 (19.44MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							2,
							OutFreq_19440k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 3 (10MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							3,
							OutFreq_10000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 4 (1PPS) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							4,
							OutFreq_1,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 6 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							6,
							OutFreq_25000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 7 (156.25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							7,
							OutFreq_156250k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 9 (2.048MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							9,
							OutFreq_2048k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 10 (155.52MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							10,
							OutFreq_155520k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 11 (77.76MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							11,
							OutFreq_77760k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);
	}

    /* Switch to backup APLL configuration now it's configured Note:
         * this step is not required if you specify APLL configuration
         * in IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or
         * APLL_CONFIG1
         */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_1);
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_2);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 1, 1);
    IDTOutput_SetOutputEnable(hdlr, 2, 1);
    IDTOutput_SetOutputEnable(hdlr, 3, 1);
    IDTOutput_SetOutputEnable(hdlr, 4, 1);
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
    IDTOutput_SetOutputEnable(hdlr, 9, 1);
    IDTOutput_SetOutputEnable(hdlr, 10, 1);
    IDTOutput_SetOutputEnable(hdlr, 11, 1);
}

/**
 * @brief Sample configuration for GPS application
 *
 *	Inputs: IN3 (10MHz), IN4 (10MHz), EX_SYNC1 (1PPS), EX_SYNC2 (1PPS)
 *	T0 Profile: BITS / SSU (Wideband) - default
 *	Outputs: OUT1 (25MHz), OUT2 (19.44MHz), OUT3 (10MHz), OUT4 (1PPS),
 *	         OUT6 (25MHz), OUT7 (156.25MHz), OUT9 (1.544MHz), OUT10 (155.52MHz),
 *	         OUT11 (77.76MHz), FRSYNC_8K_1PPS (1PPS),  MFRSYNC_2K_1PPS (1PPS)
 *	APLL1 XTAL1 =  25MHz & APLL2 XTAL2 = 24.8832MHz
 *
 * @param hdlr Device driver handler
 */
void IDTSample_GPS(T_idtDrvHdlr hdlr)
{
	/* Enable automatic selection on both T0 and T4 DPLL */
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_1, 1);
	IDTDpll_SetAutoSelInput(hdlr, DPLL_INSTANCE_2, 1);

	/* Disable all inputs */
	IDTInput_DisableAllInputs(hdlr);

	/* Configure inputs */
	{
		/* Configure input 3, 10MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 3, 10000);

	
		/* Configure input 4, 10MHz */
		IDTHlApi_ConfigureInputFrequency(hdlr, 4, 10000);

		/* Configure EX_SYNC input to 1 pps */
		IDTInput_SetSyncFrequency(hdlr, SYNC_1PPS);
        IDTInput_SetSyncSampling(hdlr,
                                 1, /* EX_SYNC1 */
                                 externalSyncSampling_onTarget); /* 0 UI */
        IDTInput_SetSyncSampling(hdlr,
                                 2, /* EX_SYNC2 */
                                 externalSyncSampling_onTarget); /* 0 UI */
        IDTInput_SetSyncEdge(hdlr,
                             externalSyncEdge_FallingEdge); /* Align to
                                                               falling
                                                               edge */
        IDTInput_SetSyncBypass(hdlr,
                               externalSyncBypass_AutoSel); /* Auto Sync
                                                               to IN2 */
        IDTInput_SetSyncEnable(hdlr,
                               externalSyncEnable_Enable); /* Enable */
	}

    /* Configure DPLL for wideband */
    IDTHlApi_wideband(hdlr, DPLL_INSTANCE_1);

	/* Select inputs for T0 DPLL */
	{
		/* Configure input 3 with priority 4 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_1, 4);

		/* Configure input 4 with priority 5 */
		IDTInput_SetPriority(hdlr, 4, DPLL_INSTANCE_1, 5);
	}

	/* Select inputs for T4 DPLL */
	{
		/* Disable input 3 */
		IDTInput_SetPriority(hdlr, 3, DPLL_INSTANCE_2, 0);
	
		/* Disable input 4 */
		IDTInput_SetPriority(hdlr, 4, DPLL_INSTANCE_2, 0);
	}

    /* Disable all outputs */
    IDTHlApi_DisableAllOutputs(hdlr);

	/* Configure outputs */
	{
		/* Configure output 1 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							1,
							OutFreq_25000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 2 (19.44MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							2,
							OutFreq_19440k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 3 (10MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							3,
							OutFreq_10000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 4 (1PPS) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							4,
							OutFreq_1,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 6 (25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							6,
							OutFreq_25000k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 7 (156.25MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							7,
							OutFreq_156250k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 9 (1.544MHz) using T0 */
		IDTHlApi_ConfigureOutput(hdlr,
							9,
							OutFreq_1544k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_MAX,
							APLL_CLK_SEL_MAX);

		/* Configure output 10 (155.52MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							10,
							OutFreq_155520k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure output 11 (77.76MHz) using T4 */
		IDTHlApi_ConfigureOutput(hdlr,
							11,
							OutFreq_77760k,
							DPLL_INSTANCE_MAX,
							sonetGigeMode_AllFromDpll1,
							APLL_CONFIG_MAX, /* Program the backup
                                                APLL configuration */
							APLL_OUT_SIG_LVPECL,
							APLL_CLK_SEL_INTERNAL);

		/* Configure FRSYNC & MFRSYNC to 1PPS */
        IDTOutput_SetFrameSync(hdlr, frameSync_FRSYNC,
                               1, /* Enable */
                               0, /* Non-inverted */
                               1); /* Pulse */
        IDTOutput_SetFrameSync(hdlr, frameSync_MFRSYNC,
                               1, /* Enable */
                               0, /* Non-inverted */
                               1); /* Pulse */
        IDTOutput_SetFrameSyncPulseEdge(hdlr,
                                        1); /* Rising Edge */
	}

    /* Switch to backup APLL configuration now it's configured Note:
     * this step is not required if you specify APLL configuration in
     * IDTHlApi_ConfigureOutput function, i.e. APLL_CONFIG0 or
     * APLL_CONFIG1
     */
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_1);
    IDTApll_Switch2NonActiveApllConfig(hdlr, APLL_INST_2);

    /* Enable output */
    IDTOutput_SetOutputEnable(hdlr, 1, 1);
    IDTOutput_SetOutputEnable(hdlr, 2, 1);
    IDTOutput_SetOutputEnable(hdlr, 3, 1);
    IDTOutput_SetOutputEnable(hdlr, 4, 1);
    IDTOutput_SetOutputEnable(hdlr, 6, 1);
    IDTOutput_SetOutputEnable(hdlr, 7, 1);
    IDTOutput_SetOutputEnable(hdlr, 9, 1);
    IDTOutput_SetOutputEnable(hdlr, 10, 1);
    IDTOutput_SetOutputEnable(hdlr, 11, 1);
}
